import time
from datetime import datetime
from calendar import monthrange

from netforce.model import Model,fields,get_model
from netforce.access import get_active_company, set_active_company, get_active_user, set_active_user
from netforce.database import get_connection

class ReportLaborCost(Model):
    _name="clinic.report.labor.cost"
    _string="Report Labor Cost"
    _transient=True
    
    _fields={
        "date": fields.Date("Month"),
        "period_id": fields.Many2One("clinic.period.line","Period"),
        "date_from": fields.Date("From", required=True),
        "date_to": fields.Date("To", required=True),
        "branch_id": fields.Many2One("clinic.branch","Branch"),
        "cycle_id": fields.Many2One("clinic.cycle","Cycle"),
        "department_id": fields.Many2One("clinic.department","Department"),
        'report_type': fields.Selection([['cross','Cross'],['detail','Detail']],'Type'),
        'show_detail': fields.Boolean("Show Detail"),
    }

    def _get_date_from(self,context={}):
        year,month,day=time.strftime("%Y-%m-%d").split("-")
        #return '%s-%s-01'%(year,month)
        return '%s-%s-%s'%(year,month,day)

    def _get_date_to(self,context={}):
        year,month,day=time.strftime("%Y-%m-%d").split("-")
        weekday, total_day=monthrange(int(year), int(month))
        #return "%s-%s-%s"%(year,month,total_day)
        return "%s-%s-%s"%(year,month,day)

    def default_get(self,field_names=None,context={},**kw):
        defaults=context.get("defaults",{})
        date_from=defaults.get("date_from", self._get_date_from())
        date_to=defaults.get("date_to", self._get_date_to())
        report_type=defaults.get("report_type","cross")
        print('defaults ', defaults)
        yearnow=date_from.split("-")[0]
        for period in get_model('clinic.period').search_browse([['name','=',yearnow]]):
            for line in period.lines:
                if line.state=='open':
                    period_id=line.id
                    date_from=line.date_start
                    date_to=line.date_stop
                    break
        res={
            'period_id': period_id,
            'date': time.strftime("%Y-%m-%d"),
            'date_from': date_from,
            'date_to': date_to,
            'show_detail': False,
            'report_type': report_type,
        }
        return res
    
    def get_report_data(self,ids,context={}):
        company_id=get_active_company()
        user_id=get_active_user()
        if not company_id:
            set_active_company(1)
            company_id=1
        if not user_id:
            set_active_user(1)
            user_id=1
        company=get_model("company").browse(company_id)
        defaults=self.default_get(context=context)
        date_from=defaults['date_from']
        date_to=defaults['date_to']
        branch_id=defaults.get("branch_id")
        department_id=defaults.get("department_id")
        cycle_id=defaults.get("cycle_id")
        show_detail=defaults['show_detail']
        report_type=defaults['report_type']
        dom=[]
        if ids:
            obj=self.browse(ids)[0]
            date_from=obj.date_from
            date_to=obj.date_to
            branch_id=obj.branch_id.id
            department_id=obj.department_id.id
            show_detail=obj.show_detail
            report_type=obj.report_type
            cycle_id=obj.cycle_id.id
        dom.append(['date','>=',date_from])
        dom.append(['date','<=',date_to])
        dom.append(['cycle_item_id.state','=', 'validated'])
        if branch_id:
            dom.append(['branch_id','=',branch_id])
        if department_id:
            dom.append(['department_id','=',department_id])
        if cycle_id:
            dom.append(['cycle_id','=',cycle_id])
        date=date_from
        hdcases={}
        hdcase_obj=get_model("clinic.hd.case")
        dstates=dict(hdcase_obj._fields['state'].selection)
        print('dom ', dom)
        t1=datetime.now()
        for hdcase in hdcase_obj.search_browse(dom):
            dpt=hdcase.department_id
            state=hdcase.state or ''
            if dpt.id not in hdcases.keys():
                hdcases[dpt.id]={
                    'name': dpt.name,
                    'department_id': dpt.id,
                    'branch_id': dpt.branch_id.id,
                    'qty':0,
                    'state':{},
                }
            #total for each department
            if state in ('waiting_payment','paid'):
                hdcases[dpt.id]['qty']+=1
            if not hdcases[dpt.id]['state'].get(state):
                hdcases[dpt.id]['state'][state]=0
            hdcases[dpt.id]['state'][state]+=1
        t2=datetime.now()
        print('time1 ',t2-t1)
        def replace_quote(dom=""):
            dom=dom.replace("False","false")
            dom=dom.replace("True","true")
            return dom.replace("'","\"")
        st=get_model('clinic.setting').browse(1)
        vark=st.var_k or 0
        cost_per_case=st.cost_per_case or 0
        lines=[] 
        dom=[]
        dom.append(["date",">=",date_from])
        dom.append(["date","<=",date_to])
        for dpt_id, vals in hdcases.items():
            sub_lines=[]
            dom2=[['department_id','=',dpt_id]]
            qty=vals['qty'] or 0
            for state, sqty in vals['state'].items():
                ok=False
                dom3=[['state','=',state]] # last domain
                if state in ('waiting_payment','paid'):
                    ok=True
                dom4=dom+dom2+dom3
                state=dstates[state]
                sub_lines.append({
                    'name': state,
                    'qty': sqty,
                    'ok': ok,
                    'action_options': 'mode=list&search_domain=%s&tab_no=0'%replace_quote('%s'%(dom4)),
                })
            lines.append({
                'name': vals['name'],
                'branch_id': vals['branch_id'],
                'department_id': vals['department_id'],
                'qty': qty,
                'sub_lines': sub_lines,
            })
        sub_name=''
        t2=datetime.now()
        print('time2 ',t2-t1)
        if department_id:
            dpt=get_model("clinic.department").browse(department_id)
            sub_name="(%s)" % dpt.name or ""
        elif branch_id:
            branch=get_model("clinic.branch").browse(branch_id)
            sub_name="(%s)" % branch.name or ""
        total_hdcase=0
        for line in lines:
            qty=line['qty'] or 0
            total_hdcase+=qty
        dlines=[]
        nlines=[]
        ctlines=[]
        if report_type=='cross':
            dom=[
                ['lcl.date','>=',"'%s'"%date_from],
                ['lcl.date','<=',"'%s'"%date_to],
            ]
            #TODO dom below make loading data very slow
            # if we send ids to browse function directly it will fast
            if branch_id:
                dom.append(['cit.branch_id','=',branch_id])
            if department_id:
                dom.append(['cit.department_id','=',department_id])
            if cycle_id:
                dom.append(['cit.cycle_id','=',cycle_id])
            
            db=get_connection()
            sql='''
                select lcl.id from clinic_labor_cost_line as lcl 
                left join clinic_labor_cost as lc on lc.id=lcl.labor_cost_id
                left join clinic_cycle_item as cit on cit.id=lc.cycle_item_id
                where cit.state='validated'
            '''
            dom_txt=''
            for f,op,v in dom:
                dom_txt+=' and %s %s %s'%(f,op,v)
            if dom_txt:
                sql+=dom_txt
            #sql+='order by department_id.name'
            lcl_ids=[r['id'] for r in db.query(sql)]
            ddata={}
            ndata={}
            ctdata={}
            t2=datetime.now()
            print('time3 ',t2-t1)
            for line in get_model('clinic.labor.cost.line').browse(lcl_ids):
                cost_id=line.labor_cost_id
                categ=line.categ_id
                categ_name=categ.name or "unknow"
                citem=cost_id.cycle_item_id
                department=citem.department_id
                dpt_name=department.name or ''
                qty=line.qty or 0
                staff=line.staff_id
                if line.type=='doctor':
                    if dpt_name not in ddata.keys():
                        ddata[dpt_name]={
                            'qty': 0,
                            'walkin_qty': 0,
                            'department_id': department.id,
                            'branch_id':department.branch_id.id,
                            'cycle_id': cycle_id,
                        }
                    if staff.number=='walkin':
                        ddata[dpt_name]['walkin_qty']+=qty
                    else:
                        ddata[dpt_name]['qty']+=qty
                elif line.type=='nurse':
                    if categ_name not in ctdata.keys():
                        ctdata[categ_name]={
                            'amount': 0,
                            'cycle_id': cycle_id,
                            'categ_id': categ.id,
                        }
                    ctdata[categ_name]['amount']+=line.amount or 0
                    if dpt_name not in ndata.keys():
                        ndata[dpt_name]={
                            'hdcases': set(),
                            'department_id': department.id,
                            'branch_id': department.branch_id.id,
                            'cycle_id': cycle_id,
                        }
                    for hdcase in citem.hd_cases:
                        if hdcase.state in ('waiting_payment','paid'):
                            ndata[dpt_name]['hdcases'].update({hdcase.id})
            total_amount=0
            t2=datetime.now()
            print('time4 ',t2-t1)
            for categ_name,vals in ctdata.items():
                amount=vals['amount'] or 0
                categ_id=vals['categ_id']
                ct_vals={
                    'name': categ_name,
                    'amount': round(amount,0),
                    'branch_id': branch_id,
                    'categ_id': categ_id,
                    'department_id': department_id,
                    'staff_type': 'nurse',
                    'cycle_id': cycle_id,
                }
                total_amount+=ct_vals['amount']
                ctlines.append(ct_vals)
            ctlines.append({
                'name': 'รวม',
                'amount': round(total_amount,0),
                'cycle_id': cycle_id,
                'branch_id': branch_id,
                'department_id': department_id,
            })
            for dt_name,vals in ddata.items():
                qty=vals['qty'] or 0
                dpt_id=vals['department_id']
                brch_id=vals['branch_id']
                walkin_qty=vals['walkin_qty']
                cost=(walkin_qty+qty)*cost_per_case
                dom1=[
                    ['department_id','=',dpt_id],
                    ['branch_id','=',brch_id],
                    ['date','>=',date_from],
                    ['date','<=',date_to],
                ]
                dom2=dom1+[['walkin','=',True]]
                dom3=dom1+[['walkin','=',False]]
                dlines.append({
                    'name': dt_name,
                    'qty': "{0:,.2f}".format(round(qty,2)),
                    'qty2': "{0:,.2f}".format(round(walkin_qty,2)),
                    'walkin_qty': walkin_qty,
                    'total_qty': walkin_qty+qty,
                    'date': date,
                    'date_from': date_from,
                    'date_to': date_to,
                    'cost': cost,
                    'total': 0,
                    'department_id': dpt_id,
                    'branch_id': brch_id,
                    'staff_type': 'doctor',
                    'cycle_id': cycle_id,
                })
            dlines=[{
                'name': 'แพทย์',
                'qty': 'จำนวนครั้ง',
                'qty2': 'ไม่จ่ายแพทย์',
                'total_qty': 'รวมจำนวนครั้ง',
                'cost': 'เป็นยอดเงินรวม',
                'desc': True,
                'department_id': None,
                'branch_id':None,
                'staff_type': 'doctor',
                'date': date,
                'date_from': date_from,
                'date_to': date_to,
            }]+sorted(dlines,key=lambda x: x['name'])
            dom4=[
                ['date','>=',date_from],
                ['date','<=',date_to],
                ['walkin','=',False],
            ]
            if department_id:
                dom4.append(['department_id','=',department_id])
            if branch_id:
                dom4.append(['branch_id','=',branch_id])
            dom5=dom4+[['walkin','=',True]]
            dlines.append({
                'name': 'รวม',
                'qty': 0,
                'qty2': 0,
                'total_qty':0,
                'cost': 0,
                'walkin_qty': 0,
                'staff_type': 'doctor',
                'cycle_id': cycle_id,
                'branch_id': branch_id,
                'department_id': department_id,
                'date': date,
                'date_from': date_from,
                'date_to': date_to,
                'option_qty': 'mode=list&search_domain=%s&tab_no=0'%replace_quote('%s'%(dom4)),
                'option_qty2': 'mode=list&search_domain=%s&tab_no=0'%replace_quote('%s'%(dom5)),
            })
            for dline in dlines[1:-1]:
                dlines[-1]['qty']+=float((dline['qty'] or "").replace(",","")) or 0
                dlines[-1]['qty2']+=float((dline['qty2'] or "").replace(",","")) or 0
                dlines[-1]['total_qty']+=dline['total_qty'] or 0
                dlines[-1]['cost']+=dline['cost'] or 0
                dlines[-1]['walkin_qty']+=float((dline['qty2'] or "").replace(",","")) or 0
            dlines[-1]['qty']="{0:,.2f}".format(round(dlines[-1]['qty'],2)) #total
            dlines[-1]['qty2']="{0:,.2f}".format(round(dlines[-1]['qty2'],2)) #total

            nlines.append({
                'name': 'พยาบาล',
                'qty': 'จำนวนครั้ง',
                'cost': 'เป็นยอดเงินรวม',
                'total': 'รวม',
                'department_id': None,
                'branch_id': None,
                'desc': True,
                'staff_type': 'nurse',
            })
            for dt_name, vals in ndata.items():
                hdcases=vals['hdcases']
                dpt_id=vals['department_id']
                brch_id=vals['branch_id']
                total_hdcase=len(hdcases)
                nlines.append({
                    'name': dt_name,
                    'qty': total_hdcase,
                    'cost': total_hdcase*vark,
                    'department_id': dpt_id,
                    'branch_id': brch_id,
                    'total': 0,
                    'staff_type': 'nurse',
                    'cycle_id': cycle_id,
                })
            nlines=sorted(nlines, key=lambda x: x['name'])
            nlines.append({
                'name': 'รวม',
                'qty': 0,
                'cost': 0,
                'total': 0,
                'staff_type': 'nurse',
                'cycle_id': cycle_id,
                'branch_id': branch_id,
                'department_id': department_id,
            })
            for nline in nlines[1:-1]:
                nlines[-1]['qty']+=nline['qty'] or 0
                nlines[-1]['cost']+=nline['cost'] or 0
                nlines[-1]['total']+=nline['total'] or 0

        def con2float(r):
            if type(r)==type(''):
                return r
            return "{0:,.2f}".format(r) 

        def v2h(keys=[],lines=[]):
            lines2=[]
            for key in keys:
                no=0
                vals={'desc': False}
                if key=='name':
                    vals['desc']=True
                for line in lines:
                    vals[no]=con2float(line[key])
                    no+=1
                lines2.append(vals)
            return lines2

        def reformat(lines=[]):
            for line in lines:
                line['amount']=con2float(line['amount'])
            return lines
        
        t2=datetime.now()
        print('time5 ',t2-t1)
        data={
            'company_name': '%s %s' % (company.name or "", sub_name),
            'date': date,
            'date_from': date_from,
            'date_to': date_to,
            'lines': lines,
            'dlines': dlines,
            'dlines2': v2h(['name','qty','qty2','total_qty','cost'],dlines),
            'nlines': nlines,
            'nlines2': v2h(['name','qty','cost'],nlines),
            'ctlines': ctlines,
            'ctlines2': reformat(ctlines),
            'total_hdcase': total_hdcase,
            'branch_id': branch_id,
            'department_id': department_id,
            'show_detail': show_detail,
            'report_type': report_type,
        }
        set_active_company(company_id)
        set_active_user(user_id)
        return data

    def onchange_date(self,context={}):
        data=context['data']
        date=data['date']
        year,month,day=date.split("-")
        weekday, total_day=monthrange(int(year), int(month))
        data['date_from']="%s-%s-01"%(year,month)
        data['date_to']="%s-%s-%s"%(year,month,total_day)
        return data

    def onchange_branch(self,context={}):
        data=context['data']
        data['department_id']=None
        return data

    def onchange_from(self,context={}):
        data=context['data']
        data['date_to']=data['date_from']
        return data
    
    def onchange_period(self,context={}):   
        data=context['data']
        period_id=data['period_id']
        period=get_model('clinic.period.line').browse(period_id)
        data['date_from']=period.date_start
        data['date_to']=period.date_stop
        return data

ReportLaborCost.register()
